import os
import pytest
import pytest_asyncio
import asyncio
from typing import Optional
from async_icq.bot import AsyncBot
from async_icq.helpers import InlineKeyboardMarkup, KeyboardButton


ADMIN_CHAT_ID = os.getenv('ADMIN_CHAT_ID')

bot = AsyncBot(
    token=os.getenv('TOKEN'),
    url=os.getenv('API_URL'),
    loop=asyncio.new_event_loop()
)

msg_id: Optional[str] = None


@pytest_asyncio.fixture
async def prepare_bot():

    await bot.start_session()

    yield bot


@pytest_asyncio.fixture
async def prepare_msg(
        prepare_bot: AsyncBot
):
    global msg_id

    if msg_id is None:
        response = await prepare_bot.send_text(
            chatId=ADMIN_CHAT_ID,
            text='Forward Test'
        )

        resp_json = await response.json()

        msg_id = resp_json.get('msgId')

    yield msg_id


@pytest.mark.asyncio
@pytest.mark.parametrize(
    'text', [
        f'Hi, @[{ADMIN_CHAT_ID}]',
        f'```Test, @[{ADMIN_CHAT_ID}]```',
        '<code>def test():\n    pass</code>',
        f'<i>Test @[{ADMIN_CHAT_ID}]</i>'
    ],
    ids=[
        'mention',
        'backtiks with mention',
        'HTML-code',
        'HTML-italic'
    ]
)
@pytest.mark.parametrize(
    'keyboard', [
        True,
        False
    ],
    ids=[
        'with keyboard',
        'none keyboard'
    ]
)
@pytest.mark.parametrize(
    'fwd_or_reply', [
        None,
        'forward',
        'reply'
    ],
    ids=[
        'nothing',
        'with forward',
        'with reply'
    ]
)
async def test_send_text(
        prepare_bot: AsyncBot,
        prepare_msg: Optional[str],
        text: str,
        keyboard: bool,
        fwd_or_reply: str
):
    replyMsgId = None
    forwardChatId = None
    forwardMsgId = None

    if fwd_or_reply == 'forward':

        forwardChatId = ADMIN_CHAT_ID

        forwardMsgId = [
            prepare_msg
        ]

    elif fwd_or_reply == 'reply':

        replyMsgId = [
            prepare_msg
        ]
    else:
        forwardChatId = None

    if keyboard:
        markup = InlineKeyboardMarkup()

        markup.row(
            KeyboardButton(
                text='Button URL',
                url='https://mail.ru/'
            )
        )

        markup.row(
            KeyboardButton(
                text='Button callback',
                callbackData='button|callback'
            )
        )
    else:
        markup = None

    response = await prepare_bot.send_text(
        chatId=ADMIN_CHAT_ID,
        text=f'Hi, @[{ADMIN_CHAT_ID}]',
        forwardMsgId=forwardMsgId,
        forwardChatId=forwardChatId,
        replyMsgId=replyMsgId,
        inlineKeyboardMarkup=markup
    )

    resp_json = await response.json()

    assert resp_json.get('msgId')
    assert resp_json.get('ok')


@pytest.mark.asyncio
@pytest.mark.parametrize(
    'fileId', [
        "02B6g000g9Fpjf1eRmkxju61eaec281bb",
        "Q8Iq8cgZQIiiprKnuD86vk61eaed711bb",
        "zfhqNk6SKLwc0KYLPgTBPX61eaf0641bb",
        "y7ltdslLWscY1yiogtwFpx61eaee9e1bb",
        "I0006XIfaT937afKh78ISw61eabec41bb",
        "W51lurUgrOo8WjnGFJUPgs61e968121bb"
    ],
    ids=[
        "image",
        "ics",
        "pdf",
        "txt",
        "ptt",
        "log"
    ]
)
@pytest.mark.parametrize(
    "caption", [
        None,
        "test",
        "<code>test</code>",
        "```test```"
    ],
    ids=[
        'without caption',
        'basic caption',
        'html-formatted caption',
        'backtick-formatted caption'
    ]
)
@pytest.mark.parametrize(
    'keyboard', [
        True,
        False
    ],
    ids=[
        'with keyboard',
        'none keyboard'
    ]
)
@pytest.mark.parametrize(
    'fwd_or_reply', [
        None,
        'forward',
        'reply'
    ],
    ids=[
        'nothing',
        'with forward',
        'with reply'
    ]
)
async def test_send_fileId(
        prepare_bot: AsyncBot,
        prepare_msg: Optional[str],
        fileId: str,
        caption: Optional[str],
        keyboard: bool,
        fwd_or_reply: str
):
    replyMsgId = None
    forwardChatId = None
    forwardMsgId = None

    if fwd_or_reply == 'forward':

        forwardChatId = ADMIN_CHAT_ID

        forwardMsgId = [
            prepare_msg
        ]

    elif fwd_or_reply == 'reply':

        replyMsgId = [
            prepare_msg
        ]
    else:
        forwardChatId = None

    if keyboard:
        markup = InlineKeyboardMarkup()

        markup.row(
            KeyboardButton(
                text='Button URL',
                url='https://mail.ru/'
            )
        )

        markup.row(
            KeyboardButton(
                text='Button callback',
                callbackData='button|callback'
            )
        )
    else:
        markup = None

    response = await prepare_bot.send_fileId(
        chatId=ADMIN_CHAT_ID,
        fileId=fileId,
        forwardMsgId=forwardMsgId,
        forwardChatId=forwardChatId,
        replyMsgId=replyMsgId,
        inlineKeyboardMarkup=markup
    )

    resp_json = await response.json()

    assert resp_json.get('msgId')
    assert resp_json.get('ok')


@pytest.mark.asyncio
@pytest.mark.parametrize(
    "caption", [
        None,
        "test",
        "<code>test</code>",
        "```test```"
    ],
    ids=[
        'without caption',
        'basic caption',
        'html-formatted caption',
        'backtick-formatted caption'
    ]
)
@pytest.mark.parametrize(
    'keyboard', [
        True,
        False
    ],
    ids=[
        'with keyboard',
        'none keyboard'
    ]
)
@pytest.mark.parametrize(
    'fwd_or_reply', [
        None,
        'forward',
        'reply'
    ],
    ids=[
        'nothing',
        'with forward',
        'with reply'
    ]
)
async def test_send_file(
        prepare_bot: AsyncBot,
        prepare_msg: Optional[str],
        caption: Optional[str],
        keyboard: bool,
        fwd_or_reply: str
):
    replyMsgId = None
    forwardChatId = None
    forwardMsgId = None

    if fwd_or_reply == 'forward':

        forwardChatId = ADMIN_CHAT_ID

        forwardMsgId = [
            prepare_msg
        ]

    elif fwd_or_reply == 'reply':

        replyMsgId = [
            prepare_msg
        ]
    else:
        forwardChatId = None

    if keyboard:
        markup = InlineKeyboardMarkup()

        markup.row(
            KeyboardButton(
                text='Button URL',
                url='https://mail.ru/'
            )
        )

        markup.row(
            KeyboardButton(
                text='Button callback',
                callbackData='button|callback'
            )
        )
    else:
        markup = None

    response = await prepare_bot.send_file(
        chatId=ADMIN_CHAT_ID,
        file_path='pytest.ini',
        caption=caption,
        replyMsgId=replyMsgId,
        forwardChatId=forwardChatId,
        forwardMsgId=forwardMsgId,
        inlineKeyboardMarkup=markup
    )

    resp_json = await response.json()

    prepare_bot.logger.warning(f'{resp_json}')


# Импорт недавно установленного пакета setuptools.
import setuptools


import json
import urllib.request


def get_last_version():

    request = urllib.request.urlopen(
        "https://pypi.org/pypi/async-icq/json").read()

    last_releases = list(
        json.loads(request).get('releases').keys())

    last_release = sorted([
        [
            int(i) for i in last_release.split('.')
        ]
        for last_release in last_releases
    ])[-1]

    return last_release



def auto_version_increase():

    major, middle, minor = get_last_version()

    if minor < 10:
        return f'{major}.{middle}.{minor + 1}'
    else:
        minor = 0
        if middle < 10:
            return f'{major}.{middle + 1}.{minor}'
        else:
            middle = 0
            return f'{major + 1}.{middle}.{minor}'


with open("README.md", "r") as fh:
    long_description = fh.read()

# Определение requests как requirements для того, чтобы этот пакет работал.
# Зависимости проекта.
requirements = [
    "aiohttp>=3.8.1",
    "aiologger>=0.6.1",
    "aiosignal>=1.2.0",
    "async-timeout>=4.0.2",
    "attrs>=21.4.0",
    "charset-normalizer>=2.0.10",
    "frozenlist>=1.2.0",
    "idna>=3.3",
    "multidict>=5.2.0",
    "yarl>=1.7.2",
]

# Функция, которая принимает несколько аргументов.
# Она присваивает эти значения пакету.
setuptools.setup(
    # Имя дистрибутива пакета.
    # Оно должно быть уникальным,
    # поэтому добавление вашего имени пользователя
    # в конце является обычным делом.
    name="async_icq",
    # Номер версии вашего пакета.
    # Обычно используется семантическое управление версиями.
    version=auto_version_increase(),
    # Имя автора.
    author="Valerii Korobov",
    # Его почта.
    author_email="dasshit@yandex.ru",
    # Краткое описание, которое будет показано на странице PyPi.
    description="ICQ/VK Teams Bot API interface",
    # Длинное описание, которое будет отображаться на странице PyPi.
    # Использует README.md репозитория для заполнения.
    long_description=long_description,
    # Определяет тип контента, используемый в long_description.
    long_description_content_type="text/markdown",
    # URL-адрес, представляющий домашнюю страницу проекта.
    # Большинство проектов ссылаются на репозиторий.
    url="https://github.com/dasshit/async-icq",
    # Находит все пакеты внутри проекта и объединяет их в дистрибутив.
    packages=setuptools.find_packages(
        exclude=[
            "examples",
            ".github",
            ".pytest_cache"
        ]
    ),
    # requirements или dependencies,
    # которые будут установлены вместе с пакетом,
    # когда пользователь установит его через pip.
    # install_requires=requirements,
    # предоставляет pip некоторые метаданные о пакете.
    # Также отображается на странице PyPi.
    install_requires=requirements,
    classifiers=[
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Communications :: Chat",
        "Topic :: Internet",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    keywords="mailru im bot api",
    # Требуемая версия Python.
    python_requires='>=3.5',
)
